package org.netkernel.rdf.jena.endpoint;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.rdf.jena.rep.IRepJenaModel;
import org.netkernel.rdf.jena.rep.JenaModelRepresentation;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.shared.Lock;
import com.hp.hpl.jena.sparql.core.ResultBinding;

public class QueryModelAccessor extends StandardAccessorImpl
{
	private static final String ASK="jRDFQueryModelASK";
	private static final int ASK_INT=1;
	private static final String CONSTRUCT="jRDFQueryModelCONSTRUCT";
	private static final int CONSTRUCT_INT=2;
	private static final String DESCRIBE="jRDFQueryModelDESCRIBE";
	private static final int DESCRIBE_INT=3;
	private static final String SELECT="jRDFQueryModelSELECT";
	private static final int SELECT_INT=4;
	
	private static final Map<String, Integer> map=new HashMap<String, Integer>();
	
	{	map.put(ASK, ASK_INT);
		map.put(CONSTRUCT, CONSTRUCT_INT);
		map.put(DESCRIBE, DESCRIBE_INT);
		map.put(SELECT, SELECT_INT);
	}

	public QueryModelAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	IRepJenaModel jmr=context.source("arg:operand", IRepJenaModel.class);
		Model m=jmr.getModelReadOnly();
		Query q=context.source("arg:operator",Query.class);
		boolean modelflag=context.getThisRequest().argumentExists("model");
		Object result=null;
		String key=context.getThisRequest().getArgumentValue("activeType");
		Integer type=map.get(key);
		synchronized(q)
		{	QueryExecution qe=QueryExecutionFactory.create(q, m);
			Lock ml=m.getLock();
			ml.enterCriticalSection(Lock.READ);
			try
			{	switch(type)
				{	case ASK_INT:
						result=qe.execAsk();
					break;
					case CONSTRUCT_INT:
						result=new JenaModelRepresentation(qe.execConstruct());
					break;
					case DESCRIBE_INT:
						result=new JenaModelRepresentation(qe.execDescribe());
					break;
					case SELECT_INT:
						ResultSet rs=qe.execSelect();
						if(modelflag)
						{	result=new JenaModelRepresentation(ResultSetFormatter.toModel(rs));						
						}
						else
						{	result=ResultSetFormatter.asXMLString(rs);							
						}
					break;
				}
			}
			finally
			{	ml.leaveCriticalSection();				
			}
		}
		context.createResponseFrom(result);
	}
}
