package org.netkernel.rdf.jena.endpoint;

import java.util.Map;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.rdf.jena.rep.JenaDatasetRepresentation;
import org.netkernel.rdf.jena.rep.JenaModelRepresentation;

import com.github.jsonldjava.jena.JenaJSONLD;
import com.hp.hpl.jena.query.Dataset;
import com.hp.hpl.jena.query.DatasetFactory;
import com.hp.hpl.jena.rdf.arp.states.RDFCollection;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.tdb.TDBFactory;
import com.hp.hpl.jena.tdb.TDBLoader;
import com.hp.hpl.jena.tdb.store.DatasetGraphTDB;
import com.hp.hpl.jena.tdb.transaction.DatasetGraphTransaction;
import com.hp.hpl.jena.util.CollectionFactory;

import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.riot.RDFLanguages;

public class RDFParserAccessor extends StandardAccessorImpl
{
	public RDFParserAccessor()
	{	this.declareThreadSafe();
		JenaJSONLD.init();
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	Model m=ModelFactory.createDefaultModel();
		IReadableBinaryStreamRepresentation opd=context.source("arg:operand", IReadableBinaryStreamRepresentation.class);
		String base=context.getThisRequest().getArgumentValue("base"); //The Base URI from which to resolve relatives - null if doesn't exist.
		String type=context.getThisRequest().getArgumentValue("activeType");
		
		Object rep=null;
		if(type.equals("jRDFParseXML"))
		{	RDFDataMgr.read(m, opd.getInputStream(),base, RDFLanguages.RDFXML);
		}
		else if (type.equals("jRDFParseN3"))
		{	RDFDataMgr.read(m, opd.getInputStream(),base, RDFLanguages.N3);
		}
		else if (type.equals("jRDFParseN-Triple"))
		{	RDFDataMgr.read(m, opd.getInputStream(),base, RDFLanguages.NTRIPLES);
		}
		else if (type.equals("jRDFParseTURTLE"))
		{	RDFDataMgr.read(m, opd.getInputStream(),base, RDFLanguages.TURTLE);
		}
		else if (type.equals("jRDFParseJSONLD"))
		{	RDFDataMgr.read(m, opd.getInputStream(),base, JenaJSONLD.JSONLD);
		}
		else if (type.equals("jRDFParseTRIG"))
		{	Dataset ds=TDBFactory.createDataset();
			RDFDataMgr.read(ds, opd.getInputStream(),base, RDFLanguages.TRIG);
			rep=new JenaDatasetRepresentation(ds);
		}
		if(rep==null)
		{	rep=new JenaModelRepresentation(m);
		}
		context.createResponseFrom(rep);
	}
}
